@@ -249,100 +249,6 @@ def test_glean_namespace(tmp_path):
         assert content.count("import Bar.private.CounterMetricType") == 1
 
 
-def test_gecko_datapoints(tmp_path):
-    """Test translating metrics to Kotlin files."""
-    translate.translate(
-        ROOT / "data" / "gecko.yaml",
-        "kotlin",
-        tmp_path,
-        {"glean_namespace": "Bar"},
-        {"allow_reserved": True},
-    )
-
-    metrics_files = [
-        "GfxContentCheckerboard.kt",
-        "GfxInfoAdapter.kt",
-        "PagePerf.kt",
-        "NonGeckoMetrics.kt",
-        "GleanBuildInfo.kt",
-    ]
-    assert set(x.name for x in tmp_path.iterdir()) == set(
-        ["GleanGeckoMetricsMapping.kt"] + metrics_files
-    )
-
-    # Make sure descriptions made it in
-    with (tmp_path / "GleanGeckoMetricsMapping.kt").open("r", encoding="utf-8") as fd:
-        content = fd.read()
-        # Make sure we're adding the relevant Glean SDK import, once.
-        assert content.count("import Bar.private.HistogramMetricBase") == 1
-
-        # Validate the generated Gecko metric mapper Kotlin functions.
-        # NOTE: Indentation, whitespaces  and text formatting of the block
-        # below are important. Do not change them unless the file format
-        # changes, otherwise validation will fail.
-        expected_func = """    fun getHistogram(geckoMetricName: String): HistogramMetricBase? {
-        return when (geckoMetricName) {
-            // From GfxContentCheckerboard.kt
-            "CHECKERBOARD_DURATION" -> GfxContentCheckerboard.duration
-            // From PagePerf.kt
-            "GV_PAGE_LOAD_MS" -> PagePerf.loadTime
-            "GV_PAGE_RELOAD_MS" -> PagePerf.reloadTime
-            else -> null
-        }
-    }"""  # noqa
-
-        assert expected_func in content
-
-        expected_func = """    fun getCategoricalMetric(
-        geckoMetricName: String
-    ): LabeledMetricType<CounterMetricType>? {
-        return when (geckoMetricName) {
-            // From PagePerf.kt
-            "DOM_SCRIPT_PRELOAD_RESULT" -> PagePerf.domScriptPreload
-            else -> null
-        }
-    }"""
-
-        assert expected_func in content
-
-        expected_func = """    fun getBooleanScalar(geckoMetricName: String): BooleanMetricType? {
-        return when (geckoMetricName) {
-            // From GfxInfoAdapter.kt
-            "gfx_adapter.stand_alone" -> GfxInfoAdapter.standAlone
-            else -> null
-        }
-    }"""  # noqa
-
-        assert expected_func in content
-
-        expected_func = """    fun getStringScalar(geckoMetricName: String): StringMetricType? {
-        return when (geckoMetricName) {
-            // From GfxInfoAdapter.kt
-            "gfx_adapter.vendor_id" -> GfxInfoAdapter.vendorId
-            else -> null
-        }
-    }"""  # noqa
-
-        assert expected_func in content
-
-        expected_func = """    fun getQuantityScalar(geckoMetricName: String): QuantityMetricType? {
-        return when (geckoMetricName) {
-            // From GfxInfoAdapter.kt
-            "gfx_adapter.width" -> GfxInfoAdapter.screenWidth
-            else -> null
-        }
-    }"""  # noqa
-
-        assert expected_func in content
-
-    for file_name in metrics_files:
-        with (tmp_path / file_name).open("r", encoding="utf-8") as fd:
-            content = fd.read()
-            assert "HistogramMetricBase" not in content
-
-    run_linters(tmp_path.glob("*.kt"))
-
-
 def test_event_extra_keys_in_correct_order(tmp_path):
     """
     Assert that the extra keys appear in the parameter and the enumeration in
@@ -365,11 +271,12 @@ def test_event_extra_keys_in_correct_order(tmp_path):
         content = fd.read()
         content = " ".join(content.split())
         assert "ExampleExtra(" in content
+        assert "and1withextracasing:" in content
         assert "alice:" in content
         assert "bob:" in content
         assert "charlie:" in content
         assert ": EventExtras" in content
-        assert 'allowedExtraKeys = listOf("alice", "bob", "charlie")' in content
+        assert 'allowedExtraKeys = listOf("And1WithExtraCasing", "alice", "bob", "charlie")' in content
 
 
 def test_arguments_are_generated_in_deterministic_order(tmp_path):
@@ -392,7 +299,7 @@ def test_arguments_are_generated_in_deterministic_order(tmp_path):
     with (tmp_path / "Event.kt").open("r", encoding="utf-8") as fd:
         content = fd.read()
         content = " ".join(content.split())
-        expected = 'EventMetricType<ExampleExtra> by lazy { // generated from event.example EventMetricType<ExampleExtra>( CommonMetricData( category = "event", name = "example", sendInPings = listOf("events"), lifetime = Lifetime.PING, disabled = false ), allowedExtraKeys = listOf("alice", "bob", "charlie")) } }'  # noqa
+        expected = 'EventMetricType<ExampleExtra> by lazy { // generated from event.example EventMetricType<ExampleExtra>( CommonMetricData( category = "event", name = "example", sendInPings = listOf("events"), lifetime = Lifetime.PING, disabled = false ), allowedExtraKeys = listOf("And1WithExtraCasing", "alice", "bob", "charlie")) } }'  # noqa
         assert expected in content
 
 
@@ -472,6 +379,13 @@ def test_object_metric(tmp_path):
         ["ActivityStream.kt", "ComplexTypes.kt", "CrashStack.kt", "GleanBuildInfo.kt"]
     )
 
+    with (tmp_path / "ComplexTypes.kt").open("r", encoding="utf-8") as fd:
+        content = fd.read()
+        content = " ".join(content.split())
+
+        assert "typealias ArrayInArrayObjectItemItem = Boolean" in content
+        assert "typealias NumberArrayObjectItem = Int" in content
+
     with (tmp_path / "CrashStack.kt").open("r", encoding="utf-8") as fd:
         content = fd.read()
         content = " ".join(content.split())
@@ -480,11 +394,11 @@ def test_object_metric(tmp_path):
         assert "data class ThreadsObject(" in content
         assert "data class ThreadsObjectItem(" in content
         assert (
-            "var frames: ThreadsObjectItemItemFrames = ThreadsObjectItemItemFrames"
+            "var frames: ThreadsObjectItemFrames = ThreadsObjectItemFrames"
             in content
         )
 
-        assert "data class ThreadsObjectItemItemFramesItem(" in content
+        assert "data class ThreadsObjectItemFramesItem(" in content
         assert "var moduleIndex: Int? = null," in content
         assert "var ip: String? = null," in content
         assert "var trust: String? = null," in content
