@@ -4,8 +4,8 @@
 # http://creativecommons.org/publicdomain/zero/1.0/
 
 from pathlib import Path
+import json
 import re
-import sys
 import textwrap
 
 import pytest
@@ -53,10 +53,6 @@ def test_parser_invalid():
     assert "could not determine a constructor for the tag" in errors[0]
 
 
-@pytest.mark.skipif(
-    sys.version_info < (3, 7),
-    reason="Error messages have literal 'OrderedDict' in them on Python 3.6",
-)
 def test_parser_schema_violation():
     """1507792"""
     all_metrics = parser.parse_objects(ROOT / "data" / "schema-violation.yaml")
@@ -114,13 +110,13 @@ def test_parser_schema_violation():
         """,
         """
         ```
-        gleantest.short.category:very_long_metric_name_this_is_too_long_s_well
+        gleantest.short.category:very_long_metric_name_this_is_too_long_as_well_since_it_has_sooooo_many_characters
         ```
 
-        'very_long_metric_name_this_is_too_long_s_well' is not valid under any
+        'very_long_metric_name_this_is_too_long_as_well_since_it_has_sooooo_many_characters' is not valid under any
         of the given schemas
-        'very_long_metric_name_this_is_too_long_s_well' is too long
-        """,
+        'very_long_metric_name_this_is_too_long_as_well_since_it_has_sooooo_many_characters' is too long
+        """,  # noqa: E501 #
         """
         ```
         gleantest:
@@ -143,43 +139,214 @@ def test_parser_schema_violation():
             extra_keys:
                 key_1:
                 description: Sample extra key
+                type: string
                 key_2:
                 description: Sample extra key
+                type: string
                 key_3:
                 description: Sample extra key
+                type: string
                 key_4:
                 description: Sample extra key
+                type: string
                 key_5:
                 description: Sample extra key
+                type: string
                 key_6:
                 description: Sample extra key
+                type: string
                 key_7:
                 description: Sample extra key
+                type: string
                 key_8:
                 description: Sample extra key
+                type: string
                 key_9:
                 description: Sample extra key
+                type: string
                 key_10:
                 description: Sample extra key
+                type: string
                 key_11:
                 description: Sample extra key
+                type: string
+                key_12:
+                description: Sample extra key
+                type: string
+                key_13:
+                description: Sample extra key
+                type: string
+                key_14:
+                description: Sample extra key
+                type: string
+                key_15:
+                description: Sample extra key
+                type: string
+                key_16:
+                description: Sample extra key
+                type: string
+                key_17:
+                description: Sample extra key
+                type: string
+                key_18:
+                description: Sample extra key
+                type: string
+                key_19:
+                description: Sample extra key
+                type: string
+                key_20:
+                description: Sample extra key
+                type: string
+                key_21:
+                description: Sample extra key
+                type: string
+                key_22:
+                description: Sample extra key
+                type: string
+                key_23:
+                description: Sample extra key
+                type: string
+                key_24:
+                description: Sample extra key
+                type: string
+                key_25:
+                description: Sample extra key
+                type: string
+                key_26:
+                description: Sample extra key
+                type: string
+                key_27:
+                description: Sample extra key
+                type: string
+                key_28:
+                description: Sample extra key
+                type: string
+                key_29:
+                description: Sample extra key
+                type: string
+                key_30:
+                description: Sample extra key
+                type: string
+                key_31:
+                description: Sample extra key
+                type: string
+                key_32:
+                description: Sample extra key
+                type: string
+                key_33:
+                description: Sample extra key
+                type: string
+                key_34:
+                description: Sample extra key
+                type: string
+                key_35:
+                description: Sample extra key
+                type: string
+                key_36:
+                description: Sample extra key
+                type: string
+                key_37:
+                description: Sample extra key
+                type: string
+                key_38:
+                description: Sample extra key
+                type: string
+                key_39:
+                description: Sample extra key
+                type: string
+                key_40:
+                description: Sample extra key
+                type: string
+                key_41:
+                description: Sample extra key
+                type: string
+                key_42:
+                description: Sample extra key
+                type: string
+                key_43:
+                description: Sample extra key
+                type: string
+                key_44:
+                description: Sample extra key
+                type: string
+                key_45:
+                description: Sample extra key
+                type: string
+                key_46:
+                description: Sample extra key
+                type: string
+                key_47:
+                description: Sample extra key
+                type: string
+                key_48:
+                description: Sample extra key
+                type: string
+                key_49:
+                description: Sample extra key
+                type: string
+                key_50:
+                description: Sample extra key
+                type: string
+                key_51:
+                description: Sample extra key
+                type: string
         ```
-        {'key_1': {'description': 'Sample extra key'},
-        'key_2': {'description': 'Sample extra key'},
-        'key_3': {'description': 'Sample extra key'},
-        'key_4': {'description': 'Sample extra key'},
-        'key_5': {'description': 'Sample extra key'},
-        'key_6': {'description': 'Sample extra key'},
-        'key_7': {'description': 'Sample extra key'},
-        'key_8': {'description': 'Sample extra key'},
-        'key_9': {'description': 'Sample extra key'},
-        'key_10': {'description': 'Sample extra key'},
-        'key_11': {'description': 'Sample extra key'}
+        {'key_1': {'description': 'Sample extra key','type': 'string'},
+        'key_2': {'description': 'Sample extra key','type': 'string'},
+        'key_3': {'description': 'Sample extra key','type': 'string'},
+        'key_4': {'description': 'Sample extra key','type': 'string'},
+        'key_5': {'description': 'Sample extra key','type': 'string'},
+        'key_6': {'description': 'Sample extra key','type': 'string'},
+        'key_7': {'description': 'Sample extra key','type': 'string'},
+        'key_8': {'description': 'Sample extra key','type': 'string'},
+        'key_9': {'description': 'Sample extra key','type': 'string'},
+        'key_10': {'description': 'Sample extra key','type': 'string'},
+        'key_11': {'description': 'Sample extra key','type': 'string'},
+        'key_12': {'description': 'Sample extra key','type': 'string'},
+        'key_13': {'description': 'Sample extra key','type': 'string'},
+        'key_14': {'description': 'Sample extra key','type': 'string'},
+        'key_15': {'description': 'Sample extra key','type': 'string'},
+        'key_16': {'description': 'Sample extra key','type': 'string'},
+        'key_17': {'description': 'Sample extra key','type': 'string'},
+        'key_18': {'description': 'Sample extra key','type': 'string'},
+        'key_19': {'description': 'Sample extra key','type': 'string'},
+        'key_20': {'description': 'Sample extra key','type': 'string'},
+        'key_21': {'description': 'Sample extra key','type': 'string'},
+        'key_22': {'description': 'Sample extra key','type': 'string'},
+        'key_23': {'description': 'Sample extra key','type': 'string'},
+        'key_24': {'description': 'Sample extra key','type': 'string'},
+        'key_25': {'description': 'Sample extra key','type': 'string'},
+        'key_26': {'description': 'Sample extra key','type': 'string'},
+        'key_27': {'description': 'Sample extra key','type': 'string'},
+        'key_28': {'description': 'Sample extra key','type': 'string'},
+        'key_29': {'description': 'Sample extra key','type': 'string'},
+        'key_30': {'description': 'Sample extra key','type': 'string'},
+        'key_31': {'description': 'Sample extra key','type': 'string'},
+        'key_32': {'description': 'Sample extra key','type': 'string'},
+        'key_33': {'description': 'Sample extra key','type': 'string'},
+        'key_34': {'description': 'Sample extra key','type': 'string'},
+        'key_35': {'description': 'Sample extra key','type': 'string'},
+        'key_36': {'description': 'Sample extra key','type': 'string'},
+        'key_37': {'description': 'Sample extra key','type': 'string'},
+        'key_38': {'description': 'Sample extra key','type': 'string'},
+        'key_39': {'description': 'Sample extra key','type': 'string'},
+        'key_40': {'description': 'Sample extra key','type': 'string'},
+        'key_41': {'description': 'Sample extra key','type': 'string'},
+        'key_42': {'description': 'Sample extra key','type': 'string'},
+        'key_43': {'description': 'Sample extra key','type': 'string'},
+        'key_44': {'description': 'Sample extra key','type': 'string'},
+        'key_45': {'description': 'Sample extra key','type': 'string'},
+        'key_46': {'description': 'Sample extra key','type': 'string'},
+        'key_47': {'description': 'Sample extra key','type': 'string'},
+        'key_48': {'description': 'Sample extra key','type': 'string'},
+        'key_49': {'description': 'Sample extra key','type': 'string'},
+        'key_50': {'description': 'Sample extra key','type': 'string'},
+        'key_51': {'description': 'Sample extra key','type': 'string'}
         } has too many properties
         Documentation for this node:
             The acceptable keys on the "extra" object sent with events. This is an
             object mapping the key to an object containing metadata about the key.
-            A maximum of 10 extra keys is allowed.
+            A maximum of 50 extra keys is allowed.
             This metadata object has the following keys:
                 - `description`: **Required.** A description of the key.
             Valid when `type`_ is `event`.
@@ -191,7 +358,12 @@ def test_parser_schema_violation():
         for x in expected_errors
     )
 
-    assert sorted(list(found_errors)) == sorted(list(expected_errors))
+    # Compare errors 1-by-1 for better assertion message when it fails.
+    found = sorted(list(found_errors))
+    expected = sorted(list(expected_errors))
+
+    for found_error, expected_error in zip(found, expected):
+        assert found_error == expected_error
 
 
 def test_parser_empty():
@@ -303,10 +475,8 @@ def invalid_in_label(name):
 @pytest.mark.parametrize(
     "name",
     [
-        "name/with_slash",
-        "name#with_pound",
-        "this_name_is_too_long_and_shouldnt_be_used",
-        "",
+        "1" * 72,
+        "Møøse",
     ],
 )
 def test_invalid_names(location, name):
@@ -717,10 +887,20 @@ def test_historical_versions():
     Make sure we can load the correct version of the schema and it will
     correctly reject or not reject entries based on that.
     """
+
+    # No issues:
+    # * Bugs as numbers
+    # * event extra keys don't have a type
     contents = [
         {
             "$schema": "moz://mozilla.org/schemas/glean/metrics/1-0-0",
-            "category": {"metric": {"type": "event", "bugs": [42]}},
+            "category": {
+                "metric": {
+                    "type": "event",
+                    "extra_keys": {"key_a": {"description": "foo"}},
+                    "bugs": [42],
+                }
+            },
         }
     ]
     contents = [util.add_required(x) for x in contents]
@@ -730,10 +910,20 @@ def test_historical_versions():
     errors = list(all_metrics)
     assert len(errors) == 0
 
+    # 1 issue:
+    # * Bugs as numbers are disallowed
+    #
+    # events: not having a `type` is fine in version 2.
     contents = [
         {
             "$schema": "moz://mozilla.org/schemas/glean/metrics/2-0-0",
-            "category": {"metric": {"type": "event", "bugs": [42]}},
+            "category": {
+                "metric": {
+                    "type": "event",
+                    "extra_keys": {"key_a": {"description": "foo"}},
+                    "bugs": [42],
+                }
+            },
         }
     ]
     contents = [util.add_required(x) for x in contents]
@@ -789,6 +979,51 @@ def test_rates():
     )  # Hasn't been transformed to "denominator" yet
 
 
+def test_ping_ordering():
+    contents = parser.parse_objects(
+        [ROOT / "data" / "pings.yaml"],
+        config={"allow_reserved": False},
+    )
+
+    errs = list(contents)
+    assert len(errs) == 0
+
+    pings = list(contents.value["pings"].keys())
+
+    assert pings == sorted(pings)
+
+
+def test_metric_ordering():
+    all_metrics = parser.parse_objects(
+        [ROOT / "data" / "ordering.yaml"], config={"allow_reserved": False}
+    )
+
+    errs = list(all_metrics)
+    assert len(errs) == 0
+
+    category = all_metrics.value["testing.ordering"]
+
+    assert len(category.values()) > 0
+    test_metrics = [f"{m.category}.{m.name}" for m in category.values()]
+
+    # Alphabetically ordered
+    assert test_metrics == [
+        "testing.ordering.a_second_test_metric",
+        "testing.ordering.first_test_metric",
+        "testing.ordering.third_test_metric",
+    ]
+
+
+def test_tag_ordering():
+    all_metrics = parser.parse_objects(ROOT / "data" / "metric-with-tags.yaml")
+
+    errs = list(all_metrics)
+    assert len(errs) == 0
+
+    tags = all_metrics.value["telemetry"]["client_id"].metadata["tags"]
+    assert tags == sorted(tags)
+
+
 def test_text_valid():
     """
     Ensure that `text` metrics parse properly.
@@ -805,7 +1040,7 @@ def test_text_valid():
     assert all_metrics.value["valid.text"]["lifetime"].lifetime == metrics.Lifetime.ping
 
     assert all_metrics.value["valid.text"]["sensitivity"].data_sensitivity == [
-        metrics.DataSensitivity.web_activity
+        metrics.DataSensitivity.stored_content
     ]
 
 
@@ -834,3 +1069,167 @@ def test_text_invalid():
 
         if "builtin_pings" in error:
             assert compare("Built-in pings are not allowed", error)
+
+
+def test_metadata_tags_sorted():
+    all_metrics = parser.parse_objects(
+        [
+            util.add_required(
+                {
+                    "$tags": ["tag1"],
+                    "category": {"metric": {"metadata": {"tags": ["tag2"]}}},
+                }
+            )
+        ]
+    )
+    errors = list(all_metrics)
+    assert len(errors) == 0
+    assert all_metrics.value["category"]["metric"].disabled is False
+    assert all_metrics.value["category"]["metric"].metadata["tags"] == ["tag1", "tag2"]
+
+
+def test_no_lint_sorted():
+    all_objects = parser.parse_objects(
+        [
+            util.add_required(
+                {
+                    "no_lint": ["lint1"],
+                    "category": {"metric": {"no_lint": ["lint2"]}},
+                }
+            ),
+            util.add_required_ping(
+                {
+                    "no_lint": ["lint1"],
+                    "ping": {"no_lint": ["lint2"]},
+                }
+            ),
+            {
+                "$schema": parser.TAGS_ID,
+                # no_lint is only valid at the top level for tags
+                "no_lint": ["lint2", "lint1"],
+                "tag": {"description": ""},
+            },
+        ]
+    )
+    errors = list(all_objects)
+    assert len(errors) == 0
+    assert all_objects.value["category"]["metric"].no_lint == ["lint1", "lint2"]
+    assert all_objects.value["pings"]["ping"].no_lint == ["lint1", "lint2"]
+    assert all_objects.value["tags"]["tag"].no_lint == ["lint1", "lint2"]
+
+
+def test_no_internal_fields_exposed():
+    """
+    We accidentally exposed fields like `_config` and `_generate_enums` before.
+    These ended up in probe-scraper output.
+
+    We replicate the code probe-scraper uses
+    and ensure we get the JSON we expect from it.
+    """
+
+    results = parser.parse_objects(
+        [
+            util.add_required(
+                {
+                    "category": {
+                        "metric": {
+                            "type": "event",
+                            "extra_keys": {
+                                "key_a": {"description": "desc-a", "type": "boolean"}
+                            },
+                        }
+                    },
+                }
+            ),
+        ]
+    )
+    errs = list(results)
+    assert len(errs) == 0
+
+    metrics = {
+        metric.identifier(): metric.serialize()
+        for category, probes in results.value.items()
+        for probe_name, metric in probes.items()
+    }
+
+    expected = {
+        "category.metric": {
+            "bugs": ["http://bugzilla.mozilla.org/12345678"],
+            "data_reviews": ["https://example.com/review/"],
+            "defined_in": {"line": 3},
+            "description": "DESCRIPTION...",
+            "disabled": False,
+            "expires": "never",
+            "extra_keys": {"key_a": {"description": "desc-a", "type": "boolean"}},
+            "gecko_datapoint": "",
+            "lifetime": "ping",
+            "metadata": {},
+            "no_lint": [],
+            "notification_emails": ["nobody@example.com"],
+            "send_in_pings": ["events"],
+            "type": "event",
+            "version": 0,
+        }
+    }
+    expected_json = json.dumps(expected, sort_keys=True, indent=2)
+
+    out_json = json.dumps(
+        metrics,
+        sort_keys=True,
+        indent=2,
+    )
+    assert expected_json == out_json
+
+
+def test_object():
+    structure = {"type": "array", "items": {"type": "number"}}
+    contents = [{"category": {"metric": {"type": "object", "structure": structure}}}]
+
+    contents = [util.add_required(x) for x in contents]
+    all_metrics = parser.parse_objects(contents)
+    errors = list(all_metrics)
+    assert len(errors) == 0, errors
+    assert len(all_metrics.value) == 1
+    assert all_metrics.value["category"]["metric"]._generate_structure == structure
+
+
+def test_object_invalid():
+    contents = [{"category": {"metric": {"type": "object"}}}]
+
+    contents = [util.add_required(x) for x in contents]
+    all_metrics = parser.parse_objects(contents)
+    errors = list(all_metrics)
+    assert len(errors) == 1
+    assert "`object` is missing required parameter `structure`" in errors[0]
+
+    structure = {"type": "array", "items": {}}
+    contents = [{"category": {"metric": {"type": "object", "structure": structure}}}]
+    contents = [util.add_required(x) for x in contents]
+    all_metrics = parser.parse_objects(contents)
+    errors = list(all_metrics)
+    assert len(errors) == 1
+    assert "invalid or missing `type`" in errors[0]
+
+    structure = {"type": "array", "items": {"type": "unknown"}}
+    contents = [{"category": {"metric": {"type": "object", "structure": structure}}}]
+    contents = [util.add_required(x) for x in contents]
+    all_metrics = parser.parse_objects(contents)
+    errors = list(all_metrics)
+    assert len(errors) == 1
+    assert "invalid or missing `type`" in errors[0]
+
+    structure = {"type": "array", "properties": {}}
+    contents = [{"category": {"metric": {"type": "object", "structure": structure}}}]
+    contents = [util.add_required(x) for x in contents]
+    all_metrics = parser.parse_objects(contents)
+    errors = list(all_metrics)
+    assert len(errors) == 1
+    assert "`properties` not allowed in array structure" in errors[0]
+
+    structure = {"type": "object", "items": {}}
+    contents = [{"category": {"metric": {"type": "object", "structure": structure}}}]
+    contents = [util.add_required(x) for x in contents]
+    all_metrics = parser.parse_objects(contents)
+    errors = list(all_metrics)
+    assert len(errors) == 1
+    assert "`items` not allowed in object structure" in errors[0]
