@@ -132,7 +132,7 @@ def test_metric_type_name():
         expires="never",
     )
 
-    assert rust.type_name(event) == "EventMetric<NoExtra>"
+    assert rust.type_name(event) == "EventMetric<NoExtraKeys>"
 
     boolean = metrics.Boolean(
         type="boolean",
@@ -276,13 +276,13 @@ def test_object_metric(tmp_path):
 
         assert "ObjectMetric<ThreadsObject>" in content
         assert "pub struct ThreadsObjectItem { " in content
-        assert "frames: ThreadsObjectItemItemFrames, }" in content
+        assert "frames: ThreadsObjectItemFrames, }" in content
         assert (
-            "pub type ThreadsObjectItemItemFrames = "
-            "Vec<ThreadsObjectItemItemFramesItem>;" in content
+            "pub type ThreadsObjectItemFrames = "
+            "Vec<ThreadsObjectItemFramesItem>;" in content
         )
 
-        assert "pub struct ThreadsObjectItemItemFramesItem { " in content
+        assert "pub struct ThreadsObjectItemFramesItem { " in content
         assert "module_index: Option<i64>, " in content
         assert "ip: Option<String>, " in content
         assert "trust: Option<String>, " in content
