@@ -288,9 +288,9 @@ def test_event_extra_keys_in_correct_order(tmp_path):
         content = " ".join(content.split())
         assert (
             "struct ExampleExtra: EventExtras "
-            "{ var alice: String? var bob: String? var charlie: String?" in content
+            "{ var and1withextracasing: Bool? var alice: String? var bob: String? var charlie: String?" in content
         )
-        assert ', ["alice", "bob", "charlie"]' in content
+        assert ', ["And1WithExtraCasing", "alice", "bob", "charlie"]' in content
 
 
 def test_event_extra_keys_with_types(tmp_path):
@@ -363,15 +363,18 @@ def test_object_metric(tmp_path):
         content = fd.read()
         content = " ".join(content.split())
 
+        assert "typealias ArrayInArrayObjectItemItem = Bool" in content
+        assert "typealias NumberArrayObjectItem = Int64" in content
+
         assert "ObjectMetricType<ThreadsObject>" in content
         assert "typealias ThreadsObject = [ThreadsObjectItem]" in content
         assert "struct ThreadsObjectItem: Codable, Equatable, ObjectSerialize {" in content
         assert (
-            "var frames: ThreadsObjectItemItemFrames"
+            "var frames: ThreadsObjectItemFrames"
             in content
         )
 
-        assert "struct ThreadsObjectItemItemFramesItem: Codable, Equatable, ObjectSerialize {" in content
+        assert "struct ThreadsObjectItemFramesItem: Codable, Equatable, ObjectSerialize {" in content
         assert "var moduleIndex: Int64?" in content
         assert "var ip: String?" in content
         assert "var trust: String?" in content
